#include "Tlist.h"

/// This function creates a new Tlist.  It returns a pointer to that 
/// list (Tlist *).

Tlist * Tlist_new () {
   Tlist * list = NEW(Tlist);
   list->head = NULL;
   list->tail = NULL;
   list->count = 0;
   return list;
}

/// This function deletes a Tlist.  It frees all the memory allocated
/// to that Tlist.  

void Tlist_delete (Tlist * list) {
   Tlist_element * del;
   list->current = NULL;
   while (list->current != list->head) {
      if (list->current == NULL) list->current = list->head;
      del = list->current;
//      free (Tlist_current (list));  
      Tlist_next (list);
      free (del);                    
   }
   list->head=NULL;
   list->tail=NULL;
}

/// This function deletes a Tlist like Tlist_delete, but it also deletes the
/// data that each list entry points to.  Obviously, this function can
/// only be used if all the data was malloced.

void Tlist_delete_all (Tlist * list) {
   Tlist_element * del;
   list->current = NULL;
   while (list->current != list->head) {
      if (list->current == NULL) list->current = list->head;
      del = list->current;
      free (Tlist_current (list));  
      Tlist_next (list);
      free (del);                    
   }
   list->head=NULL;
   list->tail=NULL;
}

/// This function pushes a data pointer to the back of the Tlist.

void Tlist_push_back (Tlist * list, void * data) {
   Tlist_element * c = NEW(Tlist_element);
   if (list->tail == NULL) {
      list->tail= c;
      list->head = c;
      c->previous = c;
      c->next = c;
   }
   else {
      list->tail->next = c;
      list->head->previous = c;
      c->previous = list->tail;
      c->next = list->head;
   }
   c->data = data;
   list->tail=c;
   list->count=list->count + 1;
   list->current = c;
}

/// This function pushes a data pointer to the front of the Tlist.

void Tlist_push_front (Tlist * list, void * data) {
   Tlist_element * c = NEW(Tlist_element);
   if (list->head != NULL) list->head->previous = c;
   else list->tail = c;
   if (list->current == NULL) list->current = c;
   list->head=c;
   list->count = list->count + 1;
}

/// This function returns the data pointer associated with the current 
/// position in the Tlist.

void* Tlist_current (Tlist * list) {
   return (list->current->data);
}

/// This function increments the current position in the Tlist and returns the
/// next data pointer in the list.  That is to say it increments and then
/// returns the "new" current data pointer.

void* Tlist_next (Tlist * list) {
   if (list->current == NULL) return(NULL);
   list->current = list->current->next;
   return (list->current->data);
}

/// This function does the same thing as Tlist_next, but it wont increment
/// past the list tail.  When this function fails to increment past the tail
/// it returns NULL, which is a signal that it has reached the end of the
/// list.

void* Tlist_next_until_tail (Tlist * list) {
   if (list->current == NULL) return(NULL);
   if (list->current == list->tail) return(NULL);
   list->current = list->current->next;
   return (list->current->data);
}

/// This function decrements the current position in the Tlist and returns the
/// "next" data pointer in the list.  That is to say it decrements and then
/// returns the "new" current data pointer.
				      
void* Tlist_previous (Tlist * list) {
   if (list->current == NULL) return(NULL);
   list->current = list->current->previous;
   return (list->current->data);
}

/// This function does the same thing as Tlist_previous, but it wont 
/// decrement past the list head.  When this function fails to decrement 
/// past the head it returns NULL, which is a signal that it has reached 
/// the beginning of the list.

void* Tlist_previous_until_head (Tlist * list) {
   if (list->current == NULL) return(NULL);
   if (list->current == list->head) return(NULL);
   list->current = list->current->previous;
   return (list->current->data);
}

/// This function returns the number of items in the Tlist.

int Tlist_count (Tlist * list) {
   return (list->count);
}

/// This function returns a pointer to the data in the first list element.  
/// This function also sets the current list position to the head.

void* Tlist_head (Tlist * list) {
   if (list->head == NULL) return (NULL);
   list->current = list->head;
   return (list->head->data);  
}

/// This function returns a pointer to the data in the last list element.  
/// This function also sets the current list position to the tail.

void* Tlist_tail (Tlist * list) {
   if (list->tail == NULL) return (NULL);
   list->current = list->tail;
   return (list->tail->data);  
}

/// Returns a pointer to the data in the Nth list element.  The current
/// list position is also set to N.

void* Tlist_at (Tlist * list, int i) {
   int j=0;
   void * data;
   list->current = list->head;
   while ((list->current != list->tail) && (i != j)) {
      data = Tlist_next(list);
      j=j+1;
   }
   return(data);
}

/// This function sets up a new list head that points to existing list
/// data.  This is useful if you need to use the current iteration
/// routines at several places in a list at once.  This function
/// needs to be "recalled" any time changes in the list order or
/// count in the original list are made.

void Tlist_virtual (Tlist * lista, Tlist * listb) {
   lista->head = listb->head;
   lista->tail = listb->tail;
   lista->current = listb->current;
   lista->count = listb->count;
}

/*
int main () {
   int aa=1, ab=2, ac=3, ad=4, ae=5, af=6;
   int * g;
   Tlist * a;
   a = Tlist_new ();
   printf("New list created successfully.\n");
   Tlist_push_back (a, &aa);
   Tlist_push_back (a, &ab);
   Tlist_push_back (a, &ac);
   printf("Pushed elements onto list.\n");
   g = Tlist_tail (a);
   printf ("head = %d\n", *g);
   Tlist_delete (a);
   return(0);
}

*/
