#include "Tlist.h"
#include "Twordlist.h"
#include "Tlookuplist.h"

/// Create a new word list. (really not necessary)

Tlist * Tword_list_new () {
   Tlist * list = Tlist_new ();
   return (list);
}

/// Safely deletes a list and all the memory it allocates.

void Tword_list_delete (Tlist * list) {
   Tword * c;
   c = Tlist_head(list);
   while (c!=NULL) {
      free(c->string);
      Tlist_delete_all(c->proximity_w);
      Tlist_delete_all(c->proximity_s);
      Tlist_delete_all(c->gender_list);
      Tlist_delete_all(c->number_list);
      c = Tlist_next_until_tail(list);
   }
   Tlist_delete_all (list);
}

/// Push a word onto the word list and allocate memory for 
/// other record keeping sub-lists.

void Tword_list_push_back (Tlist * list, char *s, int sentence, int word, int pronoun, int candidate) {
   char * t;
   Tlist * d;
   Tword * c = NEW(Tword);
   c->pronoun = 0;
   c->candidate = 0;
   c->sentence = sentence;
   c->word = word;
   t = (char *)malloc(sizeof(char)*strlen(s));
   strcpy(t,s);
   c->string = t;
   d = Tlist_new ();
   c->proximity_w = d;
   d = Tlist_new ();
   c->proximity_s = d;
   d = Tlist_new ();
   c->gender_list = d;
   d = Tlist_new ();
   c->number_list = d;
   Tlist_push_back (list, c);
}

/// Push an integer item onto a sub list (for record keeping).

void Tword_list_sublist_push_back (Tlist * list, int data) {
   int * t;
   t = NEW(int); // (char *)malloc(sizeof(int));
   *t = data;
   Tlist_push_back (list, t);
}

/// Read a file into the word list.  It uses simple plain
/// text.  It doesn't ignore any special tags, etc.

void Tword_list_read_file (Tlist * list, char *filename) {
   FILE *in;
   char buffer[80];
   int ch;
   int i=0;
   int word=0;
   int sentence=0;
   int pronoun=0;
   int candidate=0;
   int done = 0;
   
   in = fopen(filename, "r");
   if (in == NULL) {
      printf ("Cannot open \"%s\".\n", filename);
      exit(8);
   }
   while (done == 0) {
      if (i>79) {
	 printf ("This file seems to contain words longer than 80 characters long.\n"
		 "Check to make sure your input file is valid plain text.\n\n");
	 exit(8);
      }
      ch = fgetc(in);
      switch (ch) {
       case '\n':
       case '\r':
       case '\t':
       case ' ':
	 if (i>0) {
	    buffer[i] = 0;
	    Tword_list_push_back(list, &buffer[0], sentence, word, pronoun, candidate);
	    word=word+1;
	    i=0;
	 }
	 break;
       case '.':
	 if (i>0) {
	    buffer[i] = 0;
	    Tword_list_push_back(list, &buffer[0], sentence, word, pronoun, candidate);
	    word=word+1;
	    sentence=sentence+1;
	    i=0;
	 }
	 break;
       case EOF:
	 if (i>0) {
	    buffer[i] = 0;
	    Tword_list_push_back(list, &buffer[0], sentence, word, pronoun, candidate);
	 }
	 done = 1;
	 break;
       default:
	 buffer[i] = ch;
	 i=i+1;
	 break;
      }
      if (ch == EOF) break;
   }
   fclose(in);
}

/// Print the data structures out in a lisp like format.
/// This is useful for debugging, and might be useful
/// as an intermediary format, if you wanted to process
/// this information further with another application.

void Tword_list_print_lispish (Tlist * list) {
   Tword * c;
   int * t;
   c = Tlist_head (list);
   while (c != NULL) {
      printf("(\"%s\" %d %d %d %d ( ", c->string, c->sentence, c->word, c->pronoun, c->candidate);
      t = Tlist_head (c->proximity_s);
      while (t != NULL) {
	 printf("%d ", *t);
	 t = Tlist_next_until_tail (c->proximity_s);
      }
      t = Tlist_head (c->proximity_w);
      printf (") ( ");
      while (t != NULL) {
	 printf("%d ", *t);
	 t = Tlist_next_until_tail (c->proximity_w);
      }
      printf("))\n");
      c = Tlist_next_until_tail (list);
   }
}

/// Print the word list in a format suitable
/// for any terminal.

void Tword_list_print_plain (Tlist * list) {
   Tword * c;
   int * t;
   int i = 1, j = 0, bol=0;
   c = Tlist_head (list);
   while (c != NULL) {
      if (j!=c->sentence) {
	 j=c->sentence;
	 printf(".\n");
	 bol=0;
      }
      if (bol==0) bol=1;
      else printf(" ");
      printf("%s", c->string);
      if (c->type == PRONOUN) {
	 printf(" (pronoun %d)", i);
	 i=i+1;
      }
      c = Tlist_next_until_tail (list);
   }
   printf(".\n");
}

/// Print the word list in HTML.  By far the
/// most readable output if you have a web browser.

void Tword_list_print_html (Tlist * list) {
   Tword * c;
   int * t;
   int i = 1, j = 0, bol=0;
   c = Tlist_head (list);
   while (c != NULL) {
      if (j!=c->sentence) {
	 j=c->sentence;
	 printf(".\n<p>");
	 bol=0;
      }
      if (bol==0) bol=1;
      else printf(" ");
      if (c->type == PRONOUN) {
	 printf("<a href=\"#pronoun%d\">",i);
      }
      printf("%s", c->string);
      if (c->type == PRONOUN) {
	 printf("</a>");
	 i=i+1;
      }
      c = Tlist_next_until_tail (list);
   }
   printf(".\n");
}

